/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

package net.sf.l2j.gameserver.handler.usercommandhandlers;

import java.text.SimpleDateFormat;
import java.util.Date;

import javolution.text.TextBuilder;
import net.sf.l2j.gameserver.handler.IUserCommandHandler;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.network.serverpackets.NpcHtmlMessage;

/**
 * Support for clan penalty user command.
 * @author Tempy
 */
public class ClanPenalty implements IUserCommandHandler
{
    private static final int[] COMMAND_IDS = { 100 };
    private long time;
    /* (non-Javadoc)
     * @see net.sf.l2j.gameserver.handler.IUserCommandHandler#useUserCommand(int, net.sf.l2j.gameserver.model.L2PcInstance)
     */
    public boolean useUserCommand(int id, L2PcInstance activeChar)
    {
    	//3.1.3
        if (id != COMMAND_IDS[0]) return false;
        String date = "";
       
        if (activeChar.getClan() != null && activeChar.getClan().getCharPenaltyExpiryTime() != 0)
        {
        	//time = activeChar.getClanCreateExpiryTime();
        	time = activeChar.getClan().getCharPenaltyExpiryTime();
        	date = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format(new Date (time));
        }
        else if (activeChar.getClan() == null && activeChar.getClanJoinExpiryTime() != 0)
        {
        	time = activeChar.getClanJoinExpiryTime();
        	date = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format(new Date (time));
        }
        else if (activeChar.getClan() == null && activeChar.getClanCreateExpiryTime() != 0)
        {
        	time = activeChar.getClanCreateExpiryTime();
        	date = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format(new Date (time));
        }
        else 
        {
        	date = "No current penalties in effect.";
        }
        TextBuilder htmlContent = new TextBuilder("<html><body>");
        htmlContent.append("<center><table width=\"270\" border=\"0\" bgcolor=\"111111\">");
        htmlContent.append("<tr><td width=\"170\">Penalty</td>");
        htmlContent.append("<td width=\"100\" align=\"center\">Expiration Date</td></tr>");
        htmlContent.append("</table><table width=\"270\" border=\"0\">");
        htmlContent.append("<tr><td>" + date + "</td></tr>");
        htmlContent.append("</table></center>");
        htmlContent.append("</body></html>");

        NpcHtmlMessage penaltyHtml = new NpcHtmlMessage(0);
        penaltyHtml.setHtml(htmlContent.toString());
        activeChar.sendPacket(penaltyHtml);

        
        return true;
    }

    /* (non-Javadoc)
     * @see net.sf.l2j.gameserver.handler.IUserCommandHandler#getUserCommandList()
     */
    public int[] getUserCommandList()
    {
        return COMMAND_IDS;
    }
}
